import { definePgTExtension } from "@postgresql-typed/util";
import { parse, stringify } from "json-buffer";
import Keyv from "keyv";
import { hash } from "ohash";
import { deserializer } from "./util/deserializer.js";
import { serializer } from "./util/serializer.js";
export default definePgTExtension({
    meta: {
        configKey: "cache",
        name: "cache",
    },
    /* c8 ignore next 27 */
    //? Can't really test the config schema, since it's only used in the setup function
    schema: {
        namespace: {
            $default: "pgt",
            $resolve: value => (typeof value === "string" ? value : "pgt"),
        },
        ttl: {
            $default: 1000 * 60 * 15,
            $resolve: value => (typeof value === "number" ? value : 1000 * 60 * 15),
        },
        types: {
            $default: ["select"],
            $resolve: value => {
                if (!Array.isArray(value))
                    return ["select"];
                const allowed = new Set(["select", "insert", "update", "delete"]);
                value = value.filter(Boolean).map(type => type.toLowerCase());
                for (const type of value)
                    if (!allowed.has(type))
                        return ["select"];
                return value;
            },
        },
        uri: {
            $resolve: value => (typeof value === "string" ? value : undefined),
            $schema: {
                type: "string",
            },
        },
    },
    setup(resolvedOptions, manager) {
        const { uri, namespace, ttl, types } = resolvedOptions, keyv = new Keyv({
            deserialize: data => deserializer(parse(data)),
            namespace,
            serialize: data => stringify(serializer(data)),
            ttl,
            uri,
        });
        manager.hook("pgt:pre-query", async (data) => {
            //* If another hook has already set the output, don't do anything
            /* c8 ignore next 1 */
            if (data.output)
                return;
            const context = defaultContext(data.context, ttl);
            if (!context.cache?.enabled)
                return;
            if (!types.includes(data.input.query.text.trim().split(" ")[0].toLowerCase()))
                return;
            const key = context.cache?.key ?? hash(data.input, {}), foundCache = await keyv.get(key);
            if (foundCache !== undefined)
                data.output = foundCache;
            context.cache.key = key;
            context.cache.cache = foundCache;
        });
        manager.hook("pgt:post-query", async (data) => {
            const context = defaultContext(data.context, ttl);
            if (!context.cache?.enabled)
                return;
            if (!types.includes(data.input.query.text.trim().split(" ")[0].toLowerCase()))
                return;
            /* c8 ignore next 1 */
            const key = context.cache?.key ?? hash(data.input, {}), isCached = await keyv.has(key);
            if (!isCached)
                await keyv.set(key, data.output, context.cache?.ttl);
            context.cache.key = key;
            context.cache.cache = data.output;
        });
    },
});
function defaultContext(contextt, ttl) {
    const context = contextt;
    context.cache ??= {};
    context.cache.enabled ??= true;
    context.cache.ttl ??= ttl;
    return context;
}
//# sourceMappingURL=index.js.map